#include "board.h"
#include "screen.h"

#include <stdio.h>
#include <stdlib.h>

/* Spielfeld einlesen. */
void read_level(char *d, char ***f, int ***t, int *w, int *h, int *x, int *y) {

	FILE *fp = NULL;
	char ch;
	int i, j;

    if (!(fp = fopen(d, "r"))) {
        fprintf(stderr, "Keine weiteren Levels mehr gefunden!\n");
        exit(1);
    }

	/* Groesse des Spielfeldes einlesen. */
    fscanf(fp, "%d %d", w, h);
	/* Newline bzw. sonstige Zeichen weglesen. */
	while ( (ch = fgetc(fp)) != '\n') {
		; 
	}

	/* Speicher fuer Spielfeld und Zielpositionen reservieren. */
	(*f) = calloc(sizeof(char *), *w);
	(*t) = calloc(sizeof(int *), *w);
	for (i = 0; i < *w; i++) {
		(*f)[i] = calloc(sizeof(char), *h);
		(*t)[i] = calloc(sizeof(int), *h);
	}

	/* Zeilenweise die Daten einlesen. */
    for (i = 0; i < *h; i++) {
        for (j = 0; j < *w; j++) {

            /* Alle Zeichen einer Zeile lesen. */
            (*f)[j][i] = fgetc(fp);

			/* Position des Weihnachtsmannes bestimmen. */
			if ((*f)[j][i] == '.') {
				*x = j;
				*y = i;
				(*f)[j][i] = ' ';
			}

			/* Zielpositionen bestimmen. */
			if ((*f)[j][i] != '_') {
				(*t)[j][i] = 0;
			}
			else {
				(*f)[j][i] = ' ';
				(*t)[j][i] = 1;
			}
        }
        /* Newline einer Zeile lesen. */
        ch = fgetc(fp);
    }
	fclose(fp);
}

/* Spielfeld ausgeben. */
void print_board(char **f, int w, int h) {
	int i, j;
	for (i = 0; i < h; i++) {
		for (j = 0;  j < w; j++) {
			printf("%c", f[j][i]);
		}
		printf("\n");
	}
}

/* Spielfeld farbig ausgeben. */
void print_boardcolored(char **f, int **t, int w, int h, int x, int y, int l) {
	int i, j;

	clear_screen();

	for (i = 0; i < h; i++) {
		set_cursor(0, i);
		for (j = 0;  j < w; j++) {
			if (t[j][i] == 1) {
				set_bgcolor(COLOR_YELLOW);
			}
			else if (f[j][i] == '#') {
				set_bgcolor(COLOR_BLUE);
			}
			printf(" ");
			reset();
		}
		printf("\n");
	}
	
	/* Weihnachtsmann und Geschenke erst am Schluss zeichnen, */
	/* da sie sonst evtl. von den Zielpos. ueberschrieben werden. */
	for (i = 0; i < h; i++) {
		for (j = 0;  j < w; j++) {
			if ( (j == x) && (i == y) ) {
				set_bgcolor(COLOR_BLACK);
				set_cursor(j, i);
				printf(" ");
			}
			else if (f[j][i] == '*') {
				set_bgcolor(COLOR_RED);
				set_cursor(j, i);
				printf(" ");
			}
			reset();
		}
		printf("\n");
	}
	printf("\nLevel %02d\n", l);
	printf("\nSteuerung: j l i m, (q)uit\n");
}

/* Speicher fuer Brett freigeben. */
void discard(char ***f, int ***t, int w, int h) {
	int i;
	for (i = 0; i < w; i++) {
		free((*f)[i]);
		free((*t)[i]);
		(*f)[i] = NULL;
		(*t)[i] = NULL;
	}
	free(*f);
	free(*t);
	(*f) = NULL;
	(*t) = NULL;
}
